/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: fwcompiler.h,v 1.4 2001/02/01 08:48:02 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Puplic License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#ifndef _FWCOMPILER_H_
#define _FWCOMPILER_H_

#include <parser.h>
#include <tree.h>


#define FROMXMLCAST(x) ((const char *)x)
#define TOXMLCAST(x) ((xmlChar *)x)


typedef struct {

    int   num;
    char* interface;
    char* direction;
    char* action;
    char* log;
    char* comment;

    int        src_neg;
    xmlNodePtr src;
    int        dst_neg;
    xmlNodePtr dst;
    int        srv_neg;
    xmlNodePtr srv;
    
}  policy_rule;

typedef struct {

    int   num;
    char* comment;

    xmlNodePtr orig_src;
    xmlNodePtr orig_dst;
    xmlNodePtr orig_srv;
    
    xmlNodePtr transl_src;
    xmlNodePtr transl_dst;
    xmlNodePtr transl_srv;

}  nat_rule;


extern int  cmpPolicyRules(policy_rule *r1,policy_rule *r2);
extern int  cmpNatRules(nat_rule *r1,nat_rule *r2);

extern int isType(xmlNodePtr x,const char *y);


/************************************************************************
 *  Implement these functions to produce compiler for new firewall
 *  platform.
 */

extern void prologue(xmlNodePtr firewall,xmlNodePtr policy,xmlNodePtr nat);
extern void epilogue(xmlNodePtr firewall,xmlNodePtr policy,xmlNodePtr nat);


/*
 *    Process policy rule. This function is platform-dependent and
 *    should be reimplemented for new firewall platform.
 * 
 *    Each rule defined in GUI will be converted to a group of
 *    subrules if groups or multiple objects are used as rule
 *    elements. Each individual subrule then will be passed to
 *    processPolicyRule, thus guaranteeing that this function has to
 *    deal with exactly one undividable object as source, destination
 *    and service.
 *
 *    Arguments:
 *
 *    xmlNodePtr firewall      - XML node for firewall object
 *    int  subrule_no          - number of subrule 
 *    int  num_subrules        - total number of subrules
 *    policy_rule* rule_line   - structure describes subrule
 *
 * */

extern void processPolicyRule(xmlNodePtr firewall,
			      int  subrule_no,
			      int  num_subrules,
			      policy_rule* rule_line);
/*
 *   NAT rules are processed in the same way as policy
 *   rules. Arguments here have the same meaning as above
 */

extern void processNATRule(xmlNodePtr firewall,
			   int  subrule_no,
			   int  num_subrules,
			   nat_rule* rule_line);


#endif




